# Library for utilizing the Puppet REST API for certificates

require 'net/https'

module CertAPI

  #including these manually for the moment. They will need to be gotten from... somewhere? eventually.
  @ca = "33.33.33.10"
  @port = "8140"
  @environment = "production"

  @http = Net::HTTP.new(@ca, @port)
  @http.use_ssl = true
  @http.verify_mode = OpenSSL::SSL::VERIFY_NONE

  def self.resource_type(pluralize = false)
    val = "certificate_status"
    val += "es" if pluralize
    val
  end

  def self.uri(name, pluralize = false)
    "/#{@environment}/#{resource_type(pluralize)}/#{name}"
  end

  # We are forcing pson here due to issues with the certificate metadata
  # returned by the Puppet REST API. The REST API returns inconsistent
  # metadata between formats (ticket #15730), but pson was patched to
  # include the full set of metadata (ticket #15731).

  def self.data
    request = Net::HTTP::Get.new(uri('all', true), initheader = {'Accept' => 'pson'})
    response = @http.request(request)
    JSON.parse(response.body)
  end

  # Get metadata for all certificates, sorted by state and name
  def self.certificate_statuses
    data.sort_by { |certificate| [certificate['state'],certificate['name']] }
  end

  # Get metadata for certificate requests
  def self.csrs
    csrs_list = []
    certificate_statuses.each do |c|
      if c['state'] == 'requested'
       csrs_list << c
      end
    end
    csrs_list
  end

  # Get a list of certnames for all certificates
  def self.certnames
    names = []
    certificate_statuses.each do |c|
      names << c['name']
    end
    names
  end

  # Get a list of certnames for just the certificate requests
  def self.csr_names
    names = []
    csrs.each do |c|
      names << c['name']
    end
    names
  end

  # Get metadata for specified certname
  def self.cert_info(name)
    cert = []
    certificate_statuses.each do |c|
      if c['name'] == name
        cert = c
      end
    end
    cert
  end

  # Sign a certificate
  def self.add(name)
    request = Net::HTTP::Put.new(uri(name), initheader = {'Content-Type' => 'text/pson'})
    request.body = '{"desired_state":"signed"}'
    response = @http.request(request)
    puts response.inspect
    puts response.body
    response.code
  end

  # Reject a certificate request
  def self.remove(name)
    request = Net::HTTP::Delete.new(uri(name))
    response = @http.request(request)
    puts response.inspect
    puts response.body
    response.code
  end

end
