require File.join(File.dirname(__FILE__), 'certificate_helpers')

module Certificates
  class CertificateLogger
    include CertificateHelpers

    def initialize
      @log = Logger.new '/var/log/pe-console/certauth.log'
      @log.formatter = proc { |sev, datetime, prog, msg|
        "#{sev} [#{datetime.strftime "%Y-%m-%d %H:%M:%S"}] #{msg}\n" }
    end

    def log_action(userinfo, certificatename, action)
      username = userinfo.username
      role = userinfo.roles.first.name.capitalize
      begin
        @log.debug "#{role} user #{username} attempting to #{action} #{certificatename}"
        status_code = yield
        @log.debug "#{role} user #{username} finished attempt to " +
                   "#{action} #{certificatename} with HTTP code #{status_code}"

        if is_successful? status_code
          @log.info "#{role} user #{username} executed #{action} on #{certificatename}"
        end
        status_code
      rescue
        @log.error "#{role} user #{username} failed to #{action} #{certificatename}"
        raise
      end
    end
  end
end
