define(['jquery', 'sandbox'], function($, sandbox) {

  var Request = sandbox.mvc.Model({

    defaults: {
      status: 'pending'
    },

    url: 'services/certificates/api/v1/csrs/',

    idAttribute: 'name',

    /**
     * Accept the certificate signing request
     * @return {jQuery.promise}
     */
    accept: function() {
      return this._sync_status('PUT', $.proxy(this.set_accepted, this));
    },

    /**
     * Reject the certificate signing request
     * @return {jQuery.promise}
     */
    reject: function () {
      return this._sync_status('DELETE', $.proxy(this.set_rejected, this));
    },

    /**
     *
     * @param request_method {string}
     * @param model_callback {Function}
     * @return {jQuery.promise}
     * @private
     */
    _sync_status: function(request_method, model_callback) {
      var deferred = new sandbox.data.deferred();

      deferred.done(model_callback);

      $.ajax({
        url: this.url + this.id,
        type: request_method,
        success: deferred.resolve,
        error: function(resp) {
          deferred.reject(resp.responseText);
        }
      });

      return deferred.promise();
    },

    set_accepted: function() {
      this.set({
        'request_open': false,
        'status': 'accepted'
      })
    },

    set_rejected: function() {
      this.set({
        'request_open': false,
        'status': 'rejected'
      })
    },

    /**
     * As signing requests don't corrospond with a RESTy model, we don't want to
     * call the server on model creation/destruction, so we override the sync() method to a noop.
     */
    sync: function() {}

  });
  
  return Request;

});
