define(['jquery', 'sandbox', 'text!../templates/request.html'],
  function($, sandbox, template) {

  sandbox.template.engine.registerHelper('truncated_fingerprint', function() {
    var text = this.fingerprint,
        length = 12;

    if (!text) {
      return '';
    }

    return text.substr(0, length - 1) + (text.length > length ? '&hellip;' : '');
  });

  return sandbox.mvc.ItemView({

    tagName: 'tr',

    template: template,

    events: {
      'click .certificates-accept': 'accept_csr',
      'click .certificates-reject': 'reject_csr',
      'click .certificates-remove': 'remove_csr'
    },

    ui: {
      accept_button: '.certificates-accept',
      reject_button: '.certificates-reject',
      action_message: '.certificates-action-message'
    },

    initialize: function() {
      this.bindTo(this.model, 'change', this.render, this);
      this.bindTo(this.model, 'destroy', this.close, this);
    },

    accept_csr: function() {
      this.disable_controls();

      var fails = [$.proxy(this.enable_controls, this),
                  $.proxy(this.show_error, this, 'Could not accept this request, try again')];

      this.model.accept().fail(fails);
    },

    reject_csr: function() {
      this.disable_controls();

      var fails = [$.proxy(this.enable_controls, this),
                  $.proxy(this.show_error, this, 'Could not reject this request, try again')];

      this.model.reject().fail(fails);
    },

    remove_csr: function() {
      this.model.destroy();
    },

    disable_controls: function() {
      this.ui.accept_button.attr('disabled', 'disabled');
      this.ui.reject_button.attr('disabled', 'disabled');
    },

    enable_controls: function() {
      this.ui.accept_button.removeAttr('disabled');
      this.ui.reject_button.removeAttr('disabled');
    },

    /**
     * Show an error message and an optional description for what caused the error.
     * @param error {string}
     * @param reason {string}
     */
    show_error: function(error, reason) {

      if (reason) {
        error = error + ' (' + reason + ')';
      }

      this.ui.action_message.text(error).show();
    },

    onRender: function() {
      this.$el.addClass(this.model.get('approvable') ? 'certificate-valid-row' : 'certificate-invalid-row');
    }
  });
});